function CtrLoad(this, channel, count)
%CTRLOAD Load counter input pulse count.
%   CTRLOAD(DEVICE, CHANNEL, COUNT) resets pulse count of counter input
%   channel CHANNEL. CHANNEL may be a scalar or a vector, in which case
%   multiple channels are loaded. Channels are numbered from 1.
%
%   Example - load pulse count of the first counter input channel
%             of MF624 board to the value of 1000
%     dev = HudaqDevice('MF624');
%     CtrLoad(dev, 1, 1000)
%     Close(dev);
%
%   See also HUDAQDEVICE, CTRREAD, CTRRESET.

%   Copyright 2002-2011 HUMUSOFT s.r.o.


% validate device handle
hudaqhandle('Validate', this.H);

% scalar-expand count if needed
if numel(count)~=numel(channel)
  if numel(count)==1
    count = count(ones(size(channel)));
  else
    error('HUDAQLIB:ArgsNotMatch', 'Number of elements of CHANNEL and COUNT must match.');
  end
end

% load counter channels
for i=1:numel(channel)
  hudaqcall('HudaqCtrLoad', this.H, int32(channel(i))-1, count(i));
end
