function value = CtrRead(this, channel)
%CTRREAD Read counter input channels.
%   VALUE = CTRREAD(DEVICE, CHANNEL) reads pulse count from counter input
%   channel CHANNEL. CHANNEL may be a scalar or a vector, in which case
%   multiple channels are read. Channels are numbered from 1.
%
%   Example - read pulse count of the first counter input channel of MF624 board
%     dev = HudaqDevice('MF624');
%     CtrRead(dev, 1)
%     Close(dev);
%
%   See also HUDAQDEVICE, CTRRESET.

%   Copyright 2002-2011 HUMUSOFT s.r.o.


% validate device handle
hudaqhandle('Validate', this.H);

% read counter channels
value = zeros(1, numel(channel));
for i=1:numel(channel)
  value(i) = hudaqcall('HudaqCtrRead', this.H, int32(channel(i))-1);
end

