function value = DIRead(this, channel)
%DIREAD Read digital input channels.
%   VALUE = DIREAD(DEVICE, CHANNEL) reads a value from digital input
%   channel CHANNEL. CHANNEL may be a scalar or a vector, in which case
%   multiple channels are read. Channels are numbered from 1.
%
%   Example - read the first digital input channel of MF624 board
%     dev = HudaqDevice('MF624');
%     DIRead(dev, 1)
%     Close(dev);
%
%   See also HUDAQDEVICE, DIREADBIT, DOWRITE.

%   Copyright 2002-2007 HUMUSOFT s.r.o.


% validate device handle
hudaqhandle('Validate', this.H);

% read multiple channels
if numel(channel)>1
  [result, dummy, value] = hudaqcall('HudaqDIReadMultiple', this.H, numel(channel), int32(channel)-1);
  value = double(value);
  if ~strcmp(result, 'HUDAQSUCCESS')
    error('HUDAQLIB:DICannotRead', 'Cannot read digital input channels %s.', mat2str(channel));
  end

% read single channel
else
  value = hudaqcall('HudaqDIRead', this.H, int32(channel)-1);  
end

