function DOWriteBit(this, channel, bit, value)
%DOWRITEBIT Write individual bits of digital output channel.
%   DOWRITEBIT(DEVICE, CHANNEL, BIT, VALUE) writes VALUE to bit BIT of
%   digital input channel CHANNEL. Channels and bits are numbered from 1.
%   BIT may be a vector, in which case VALUE must be either a vector
%   with the same number of elements or a scalar that is written to
%   all the bits.
%
%   Example - write 1 to the third bit of the first digital output channel
%     dev = HudaqDevice('MF624');
%     DOWriteBit(dev, 1, 3, 1)
%     Close(dev);
%
%   See also HUDAQDEVICE, DOWRITE, DIREADBIT.

%   Copyright 2002-2007 HUMUSOFT s.r.o.


% validate device handle
hudaqhandle('Validate', this.H);

% write multiple bits
if numel(bit)>1
  value = expandarg(value, bit) ~= 0;
  masks = bitshift(1, uint32(bit)-1);
  hudaqcall('HudaqDOWriteMultipleBits', this.H, int32(channel)-1, sum(masks), sum(masks(:)'*value(:)));

% write single bit
else
  hudaqcall('HudaqDOWriteBit', this.H, int32(channel)-1, bit-1, value);
end

