function value = GetParameter(this, channel, param)
%GETPARAMETER Get configuration parameter of a channel.
%   GETPARAMETER(DEVICE, CHANNEL, PARAM) gets parameter PARAM
%   of channel CHANNEL. Parameter names and their values
%   are documented in product documentation.
%
%   Example - read filter status on first encoder input channel:
%     dev = HudaqDevice('MF624');
%     GetParameter(dev, 1, 'HudaqEncFILTER')
%     Close(dev);
%
%   See also HUDAQDEVICE, SETPARAMETER, TRANSLATEPARAMETER.

%   Copyright 2002-2009 HUMUSOFT s.r.o.


% validate device handle
hudaqhandle('Validate', this.H);

% retrieve parameters in a loop
value = zeros(1, numel(channel));
for i=1:numel(channel)
  value(i) = hudaqcall('HudaqGetParameter', this.H, int32(channel(i))-1, extractparam(param, i));
end  

