function value = QueryRange(this, subsystem)
%QUERYRANGE Query I/O range for a HUDAQLIB device subsystem.
%   TABLE = QUERYRANGE(DEVICE, SUBSYSTEM) returns I/O range
%   table for a given HUDAQLIB device subsystem. Valid subsystem
%   values are 'HudaqAI' or 'HudaqAO'. The returned table is a structure
%   array with fields 'Hi' and 'Lo'.
%
%   Example - query analog input ranges for MF624 board:
%     dev = HudaqDevice('MF624');
%     table = QueryRange(dev, 'HudaqAI')
%     Close(dev);
%
%   See also HUDAQDEVICE, SETPARAMETER, GETPARAMETER.

%   Copyright 2002-2009 HUMUSOFT s.r.o.


% validate device handle
hudaqhandle('Validate', this.H);

% query ranges in a loop
ii = 0;
value = [];
while true
  item = hudaqcall('HudaqQueryRange', this.H, subsystem, ii);
  if item.Lo==0 && item.Hi==0           % check for the last NULL item
    break;
  end
  value = [value item];  %#ok<AGROW> we don't know the size in advance
  ii = ii+1;
end

