/* $Revision:  $ $Date:  $ */
/* dahsmf624.c - xPC Target, non-inlined S-function driver for Humusoft MF624 D/A section */
/* Copyright (c) 2003-2006 by Humusoft s.r.o. All Rights Reserved. */

#define S_FUNCTION_LEVEL 2
#define S_FUNCTION_NAME  dahsmf624

#include 	<stddef.h>
#include 	<stdlib.h>
#include 	<math.h>

#include 	"simstruc.h"

#ifdef 		MATLAB_MEX_FILE
#include 	"mex.h"
#else
#include 	<windows.h>
#include 	"io_xpcimport.h"
#include    "pci_xpcimport.h"
#endif

/* Input Arguments */
#define NUM_PARAMS             (5)
#define SLOT_ARG               (ssGetSFcnParam(S,0))
#define DEV_ARG                (ssGetSFcnParam(S,1))
#define CHANNEL_ARG            (ssGetSFcnParam(S,2))
#define RANGE_ARG              (ssGetSFcnParam(S,3))
#define SAMPLE_TIME_PARAM      (ssGetSFcnParam(S,4))

/* Convert S Function Parameters to Varibles */

#define SLOT                   ((uint_T) mxGetPr(SLOT_ARG)[0])
#define DEVICE                 ((uint_T) mxGetPr(DEV_ARG)[0])
#define SAMPLE_TIME            ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[0])
#define SAMPLE_OFFSET          ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[1])
#define SAMP_TIME_IND          (0)

#define NO_I_WORKS             (3)
#define BASE_CHIPSET_I_IND     (0)
#define BASE_ADC_I_IND         (1)
#define BASE_ALTERA_I_IND      (2)

#define NO_R_WORKS             (0)

static char_T msg[256];


#ifndef MATLAB_MEX_FILE
static void DA_Output(SimStruct *S, uint_T device, uint_T channel, real_T value)
{
    real_T out;
    volatile unsigned int *base_chipset;
    volatile unsigned short *base_adc;

    base_chipset  = (void *) ssGetIWorkValue(S, BASE_CHIPSET_I_IND);
    base_adc = (void *) ssGetIWorkValue(S, BASE_ADC_I_IND);

    out=8192*(1-value/-10);

    out=max(out,0);
    out=min(out,16383);
    out=floor(out+0.5);

    base_adc[0x10+channel] =(uint_T)out;
}
#endif

/*====================*
 * S-function methods *
 *====================*/

static void mdlCheckParameters(SimStruct *S)
{
}

static void mdlInitializeSizes(SimStruct *S)
{
    uint_T i;

#ifndef MATLAB_MEX_FILE
#include "io_xpcimport.c"
#include "pci_xpcimport.c"
#endif

    ssSetNumSFcnParams(S, NUM_PARAMS);
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, mxGetNumberOfElements(CHANNEL_ARG))) return;

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        ssSetInputPortWidth(S, i, 1);
    }

    if (!ssSetNumOutputPorts(S, 0)) return;

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, NO_R_WORKS);
    ssSetNumIWork(S, NO_I_WORKS);
    ssSetNumPWork(S, 0);

    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetSFcnParamNotTunable(S,0);
    ssSetSFcnParamNotTunable(S,1);
    ssSetSFcnParamNotTunable(S,2);
    ssSetSFcnParamNotTunable(S,3);
    ssSetSFcnParamNotTunable(S,4);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE | SS_OPTION_PLACE_ASAP);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, SAMPLE_TIME);
    ssSetOffsetTime(S, 0, SAMPLE_OFFSET);
}


#define MDL_START
static void mdlStart(SimStruct *S)
{

#ifndef MATLAB_MEX_FILE
    uint_T i;
    PCIDeviceInfo pciinfo;
    void *Physical1, *Physical0;
    void *Virtual1, *Virtual0;
    volatile unsigned int *base_chipset;
    volatile unsigned short *base_adc;
    char devName[20];
    int  devId;

    switch ((int_T)mxGetPr(DEV_ARG)[0]) {
        case 1:
            strcpy(devName,"Humusoft MF624");
            devId=0x0624;
            break;
        case 2:
            strcpy(devName,"Humusoft AD622");
            devId=0x0622;
            break;
         }

    if ((int_T)mxGetPr(SLOT_ARG)[0]<0) {
        // look for the PCI-Device
        if (rl32eGetPCIInfo(0x186C,(unsigned short)devId,&pciinfo)) {
            sprintf(msg,"%s: board not present", devName);
            ssSetErrorStatus(S,msg);
            return;
        }
    } else {
        int_T bus, slot;
        if (mxGetN(SLOT_ARG) == 1) {
            bus  = 0;
            slot = (int_T)mxGetPr(SLOT_ARG)[0];
        } else {
            bus = (int_T)mxGetPr(SLOT_ARG)[0];
            slot = (int_T)mxGetPr(SLOT_ARG)[1];
        }
        // look for the PCI-Device
        if (rl32eGetPCIInfoAtSlot(0x186C,(unsigned short)devId,(slot & 0xff) | ((bus & 0xff)<< 8),&pciinfo)) {
            sprintf(msg,"%s (bus %d, slot %d): board not present",devName, bus, slot );
            ssSetErrorStatus(S,msg);
            return;
        }
    }

    // show Device Information
    // rl32eShowPCIInfo(pciinfo);

    Physical0=(void *)pciinfo.BaseAddress[0];
    Virtual0 = rl32eGetDevicePtr(Physical0, 32, RT_PG_USERREADWRITE);
    base_chipset=(void *)Physical0;
    Physical1=(void *)pciinfo.BaseAddress[2];
    Virtual1 = rl32eGetDevicePtr(Physical1, 128, RT_PG_USERREADWRITE);
    base_adc=(void *)Physical1;

    ssSetIWorkValue(S, BASE_CHIPSET_I_IND, (int_T)base_chipset);
    ssSetIWorkValue(S, BASE_ADC_I_IND, (int_T)base_adc);

/* Set initial outputs to zero */
    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        DA_Output(S, DEVICE, (uint_T)mxGetPr(CHANNEL_ARG)[i]-1, 0);
    }
    base_chipset[0x15] = 0x064006C0;

#endif /* MATLAB_MEX_FILE */

}

static void mdlOutputs(SimStruct *S, int_T tid)
{
#ifndef MATLAB_MEX_FILE

    InputRealPtrsType uPtrs;
    uint_T i;
    volatile unsigned int *base_chipset;

    base_chipset  = (void *) ssGetIWorkValue(S, BASE_CHIPSET_I_IND);
    base_chipset[0x15] = 0x068006C0;        // disable update

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        uPtrs = ssGetInputPortRealSignalPtrs(S,i);
        DA_Output(S, DEVICE, (uint_T)mxGetPr(CHANNEL_ARG)[i]-1, *uPtrs[0]);
    }

    base_chipset[0x15] = 0x064006C0;       // simultaneous update

#endif /* MATLAB_MEX_FILE */

}


static void mdlTerminate(SimStruct *S)
{
#ifndef MATLAB_MEX_FILE
    uint_T i;
    volatile unsigned int *base_chipset;

    base_chipset  = (void *) ssGetIWorkValue(S, BASE_CHIPSET_I_IND);
    base_chipset[0x15] = 0x068006C0;        // disable update

/* Set final outputs to zero */
    for (i=0;i < mxGetNumberOfElements(CHANNEL_ARG);i++) {
        DA_Output(S, DEVICE, (uint_T)mxGetPr(CHANNEL_ARG)[i]-1, 0);
    }

    base_chipset[0x15] = 0x064006C0;       // simultaneous update

#endif /* MATLAB_MEX_FILE */
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
