function [SampleTime, SampleOffset] = mpwmhsmf624 (flag, boardType, Channels, Sample, Slot)

% MPWMHSMF624 - InitFcn and Mask Initialization for Humusoft MF624 counter section

% Copyright (c) 2003-2006 by Humusoft s.r.o.
% $Revision:  $ $Date:  $

% Initialize Data Values
persistent pool;

if flag == 0
  pool = [];
  return;
end

MAX_CHANNEL=4;

chnDisplay='disp(''MF624\nHumusoft\n';
description='MF624';

% Get Length Channel Vector
chnLength = length(Channels);

% Label Output Ports

% Compose MaskDisplay String
chnDisplay=[chnDisplay,'PWM Output'');'];
for i = 1:chnLength
  chnDisplay = strcat(chnDisplay,'port_label(''input'', ');
  chnDisplay = strcat(chnDisplay,num2str (2*i-1));
  chnDisplay = strcat(chnDisplay,' , ''');
  chnDisplay = strcat(chnDisplay, num2str (Channels(1,i)));
  chnDisplay = strcat(chnDisplay, ' (f)'') ');
  chnDisplay = strcat(chnDisplay,'port_label(''input'', ');
  chnDisplay = strcat(chnDisplay,num2str (2*i));
  chnDisplay = strcat(chnDisplay,' , ''');
  chnDisplay = strcat(chnDisplay, num2str (Channels(1,i)));
  chnDisplay = strcat(chnDisplay, ' (dc)'') ');
end

% Set MaskDisplay String
set_param(gcb,'MaskDisplay',chnDisplay);

% Check Parameters
for i=1:chnLength
   if (Channels(i)<1 | Channels(i)>MAX_CHANNEL)
      error(['The number of channels must be in the range 1..',num2str(MAX_CHANNEL)]);
   end
end


% Check sample time

sampleLength = length (Sample);

if (sampleLength > 2)
  error ('Sample length vector cannot exceed two elements');
end

if (sampleLength == 2)
  SampleTime = Sample(1);
  SampleOffset = Sample(2);
else
  SampleTime = Sample(1);
  SampleOffset = 0;
end

% Check For Multipule instances using the same channel

boardtype=['btype',num2str(boardType)];
if Slot<0
  boardref=['ref','a'];
else
  boardref=['ref',num2str(Slot)];
end
if ~isfield(pool,boardtype)
  eval(['pool.',boardtype,'=[];']);
end
level1=getfield(pool,boardtype);
if ~isfield(level1,boardref)
  eval(['level1.',boardref,'.chUsed=zeros(1,MAX_CHANNEL);']);
end
level2=getfield(level1,boardref);
for i = 1:length(Channels)
  channel=Channels(i);
  if channel<1 | channel > MAX_CHANNEL
    error(['Elements of the channel vector must be in between 1 and ',num2str(MAX_CHANNEL)]);
  end
  if level2.chUsed(channel)==1
    error(['Channel ',num2str(channel),' already in use']);
  end
  level2.chUsed(channel)=1;
end

level1=setfield(level1,boardref,level2);
pool=setfield(pool,boardtype,level1);

%% EOF mpwmhsmf624.m
