/* $Revision:  $ $Date:  $ */
/* adhsmf624.c - xPC Target, non-inlined S-function driver for A/D section of Humusoft MF624 board  */
/* Copyright (c) 2003-2006 by Humusoft s.r.o. All Rights Reserved. */

#define 	S_FUNCTION_LEVEL 	2
#define 	S_FUNCTION_NAME 	adhsmf624

#include 	<stddef.h>
#include 	<stdlib.h>

#include 	"simstruc.h"

#ifdef 		MATLAB_MEX_FILE
#include 	"mex.h"
#else
#include    <windows.h>
#include    "io_xpcimport.h"
#include    "pci_xpcimport.h"
#endif


/* Input Arguments */
#define NUM_PARAMS             (5)
#define SLOT_ARG               (ssGetSFcnParam(S,0))
#define DEV_ARG                (ssGetSFcnParam(S,1))
#define CHANNEL_ARG            (ssGetSFcnParam(S,2))
#define RANGE_ARG              (ssGetSFcnParam(S,3))
#define SAMPLE_TIME_PARAM      (ssGetSFcnParam(S,4))

/* Convert S Function Parameters to Varibles */

#define SLOT                   ((uint_T) mxGetPr(SLOT_ARG)[0])
#define SAMPLE_TIME            ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[0])
#define SAMPLE_OFFSET          ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[1])
#define SAMP_TIME_IND          (0)

#define NO_I_WORKS             (3)
#define BASE_CHIPSET_I_IND     (0)
#define BASE_ADC_I_IND         (1)
#define BASE_ALTERA_I_IND      (2)

#define NO_R_WORKS             (0)

static char_T msg[256];


/*====================*
 * S-function methods *
 *====================*/

static void mdlCheckParameters(SimStruct *S)
{
}

static void mdlInitializeSizes(SimStruct *S)
{
    uint_T i;

#ifndef MATLAB_MEX_FILE
#include "io_xpcimport.c"
#include "pci_xpcimport.c"
#endif


    ssSetNumSFcnParams(S, NUM_PARAMS);
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);



    if (!ssSetNumInputPorts(S, 0)) return;

    if (!ssSetNumOutputPorts(S, mxGetNumberOfElements(CHANNEL_ARG))) return;

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        ssSetOutputPortWidth(S, i, 1);
    }

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, NO_R_WORKS);
    ssSetNumIWork(S, NO_I_WORKS);
    ssSetNumPWork(S, 0);

    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetSFcnParamNotTunable(S,0);
    ssSetSFcnParamNotTunable(S,1);
    ssSetSFcnParamNotTunable(S,2);
    ssSetSFcnParamNotTunable(S,3);
    ssSetSFcnParamNotTunable(S,4);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE | SS_OPTION_PLACE_ASAP);

}



static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, SAMPLE_TIME);
    ssSetOffsetTime(S, 0, SAMPLE_OFFSET);
}



#define MDL_START
static void mdlStart(SimStruct *S)
{

#ifndef MATLAB_MEX_FILE


    uint_T i, channel;
    PCIDeviceInfo pciinfo;
    void *Physical2, *Physical1, *Physical0;
    void *Virtual2, *Virtual1, *Virtual0;
    volatile unsigned int *base_chipset;
    volatile unsigned short *base_adc;
    volatile unsigned int *base_altera;
    char devName[20];
    int  devId;
    int  altera_code;
    static volatile unsigned dummy;

    switch ((int_T)mxGetPr(DEV_ARG)[0]) {
        case 1:
            strcpy(devName,"Humusoft MF624");
            devId=0x0624;
            altera_code = 1;                 // Standard Altera code
            break;
        case 2:
            strcpy(devName,"Humusoft AD622");
            devId=0x0622;
            altera_code = 0;                 // Altera not present
            break;
         }

    if ((int_T)mxGetPr(SLOT_ARG)[0]<0) {
        // look for the PCI-Device
        if (rl32eGetPCIInfo(0x186C,(unsigned short)devId,&pciinfo)) {
            sprintf(msg,"%s: board not present", devName);
            ssSetErrorStatus(S,msg);
            return;
        }
    } else {
        int_T bus, slot;
        if (mxGetN(SLOT_ARG) == 1) {
            bus  = 0;
            slot = (int_T)mxGetPr(SLOT_ARG)[0];
        } else {
            bus = (int_T)mxGetPr(SLOT_ARG)[0];
            slot = (int_T)mxGetPr(SLOT_ARG)[1];
        }
        // look for the PCI-Device
        if (rl32eGetPCIInfoAtSlot(0x186C,(unsigned short)devId,(slot & 0xff) | ((bus & 0xff)<< 8),&pciinfo)) {
            sprintf(msg,"%s (bus %d, slot %d): board not present",devName, bus, slot );
            ssSetErrorStatus(S,msg);
            return;
        }
    }

    // show Device Information
    // rl32eShowPCIInfo(pciinfo);

    Physical0=(void *)pciinfo.BaseAddress[0];
    Virtual0 = rl32eGetDevicePtr(Physical0, 32, RT_PG_USERREADWRITE);
    base_chipset=(void *)Physical0;
    Physical1=(void *)pciinfo.BaseAddress[2];
    Virtual1 = rl32eGetDevicePtr(Physical1, 128, RT_PG_USERREADWRITE);
    base_adc=(void *)Physical1;
    if (altera_code != 0) {
        Physical2=(void *)pciinfo.BaseAddress[4];
        Virtual2 = rl32eGetDevicePtr(Physical2, 128, RT_PG_USERREADWRITE);
        base_altera=(void *)Physical2;
    }
    ssSetIWorkValue(S, BASE_CHIPSET_I_IND, (int_T)base_chipset);
    ssSetIWorkValue(S, BASE_ADC_I_IND, (int_T)base_adc);
    ssSetIWorkValue(S, BASE_ALTERA_I_IND, (int_T)base_altera);

    /* Check That The Hardware is functioning */

    base_adc[0x00] = 0xff;        // select all channels
    dummy = base_adc[0x10];        // start conversion
    if (!(base_chipset[0x15] & 0x020000)) { /* if data ready, error */
        sprintf(msg,"%s (%d): test A/D conversion failed", devName, SLOT);
        ssSetErrorStatus(S,msg);
        return;
    }

    for (i=0;i<0x10000;i++)
        if (!(base_chipset[0x15] & 0x020000)) break;
    if (i==0x10000) {               /* if no conversion, error */
        sprintf(msg,"%s (%d): test A/D conversion failed", devName, SLOT);
        ssSetErrorStatus(S,msg);
        return;
    }

#endif

}

static void mdlOutputs(SimStruct *S, int_T tid)
{

#ifndef MATLAB_MEX_FILE

    uint_T i, n, channel;
    volatile unsigned int *base_chipset;
    volatile unsigned short *base_adc;
    volatile unsigned int *base_adc32;
    volatile unsigned int *base_altera;
    real_T *y;
    short adParameters;
    short res;
    static volatile unsigned dummy;
    unsigned int buffer32[4];
    short *buffer16;

    buffer16 = (void *) buffer32;
    base_chipset  = (void *) ssGetIWorkValue(S, BASE_CHIPSET_I_IND);
    base_adc = (void *) ssGetIWorkValue(S, BASE_ADC_I_IND);
    base_adc32 = (void *) ssGetIWorkValue(S, BASE_ADC_I_IND);
    base_altera = (void *) ssGetIWorkValue(S, BASE_ALTERA_I_IND);

    dummy = base_adc[0x10];                    // start conversion
    while (!(base_chipset[0x15] & 0x020000));  // data ready
    buffer32[0] = base_adc32[0];                   // read data
    buffer32[1] = base_adc32[0];
    buffer32[2] = base_adc32[0];
    buffer32[3] = base_adc32[0];

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
        y=ssGetOutputPortSignal(S,i);
        y[0]= ((short)(buffer16[channel] << 2))*10/32768.;
    }

#endif

}

static void mdlTerminate(SimStruct *S)
{
}


#ifdef MATLAB_MEX_FILE  /* Is this file being compiled as a MEX-file? */
#include "simulink.c"   /* Mex glue */
#else
#include "cg_sfun.h"    /* Code generation glue */
#endif


