/* $Revision:  $ $Date:  $ */
/* enchsmf624.c - xPC Target, non-inlined S-function driver for encoder section of Humusoft MF624 board  */
/* Copyright (c) 2003-2006 by Humusoft s.r.o. All Rights Reserved. */

#define 	S_FUNCTION_LEVEL 	2
#define 	S_FUNCTION_NAME 	enchsmf624

#include 	<stddef.h>
#include 	<stdlib.h>

#include 	"simstruc.h"

#ifdef 		MATLAB_MEX_FILE
#include 	"mex.h"
#else
#include 	<windows.h>
#include 	"io_xpcimport.h"
#include    "pci_xpcimport.h"
#endif


/* Input Arguments */
#define NUM_PARAMS             (6)
#define SLOT_ARG               (ssGetSFcnParam(S,0))
#define DEV_ARG                (ssGetSFcnParam(S,1))
#define CHANNEL_ARG            (ssGetSFcnParam(S,2))
#define FILTER_ARG             (ssGetSFcnParam(S,3))
#define INDEX_ARG              (ssGetSFcnParam(S,4))
#define SAMPLE_TIME_PARAM      (ssGetSFcnParam(S,5))

/* Convert S Function Parameters to Varibles */

#define SLOT                   ((uint_T) mxGetPr(SLOT_ARG)[0])
#define SAMPLE_TIME            ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[0])
#define SAMPLE_OFFSET          ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[1])
#define SAMP_TIME_IND          (0)

#define NO_I_WORKS             (3)
#define BASE_CHIPSET_I_IND     (0)
#define BASE_ADC_I_IND         (1)
#define BASE_ALTERA_I_IND      (2)

#define NO_R_WORKS             (0)

#define MAXIRCFILT 2500000

static char_T msg[256];


/*====================*
 * S-function methods *
 *====================*/

static void mdlCheckParameters(SimStruct *S)
{
}

static void mdlInitializeSizes(SimStruct *S)
{
    uint_T i;

#ifndef MATLAB_MEX_FILE
#include "io_xpcimport.c"
#include "pci_xpcimport.c"
#endif


    ssSetNumSFcnParams(S, NUM_PARAMS);
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);



    if (!ssSetNumInputPorts(S, 0)) return;

    if (!ssSetNumOutputPorts(S, mxGetNumberOfElements(CHANNEL_ARG))) return;

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        ssSetOutputPortWidth(S, i, 1);
    }

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, NO_R_WORKS);
    ssSetNumIWork(S, NO_I_WORKS);
    ssSetNumPWork(S, 0);

    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetSFcnParamNotTunable(S,0);
    ssSetSFcnParamNotTunable(S,1);
    ssSetSFcnParamNotTunable(S,2);
    ssSetSFcnParamNotTunable(S,3);
    ssSetSFcnParamNotTunable(S,4);
    ssSetSFcnParamNotTunable(S,5);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE | SS_OPTION_PLACE_ASAP);

}



static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, SAMPLE_TIME);
    ssSetOffsetTime(S, 0, SAMPLE_OFFSET);
}



#define MDL_START
static void mdlStart(SimStruct *S)
{

#ifndef MATLAB_MEX_FILE

    uint_T i, channel;
    PCIDeviceInfo pciinfo;
    void *Physical2, *Physical1, *Physical0;
    void *Virtual2, *Virtual1, *Virtual0;
    volatile unsigned int *base_chipset;
    volatile unsigned short *base_adc;
    volatile unsigned int *base_altera;
    char devName[20];
    int  devId;
    unsigned mode;
    unsigned irc_shadow;

    switch ((int_T)mxGetPr(DEV_ARG)[0]) {
        case 1:
            strcpy(devName,"Humusoft MF624");
            devId=0x0624;
            break;
         }

    if ((int_T)mxGetPr(SLOT_ARG)[0]<0) {
        // look for the PCI-Device
        if (rl32eGetPCIInfo(0x186C,(unsigned short)devId,&pciinfo)) {
            sprintf(msg,"%s: board not present", devName);
            ssSetErrorStatus(S,msg);
            return;
        }
    } else {
        int_T bus, slot;
        if (mxGetN(SLOT_ARG) == 1) {
            bus  = 0;
            slot = (int_T)mxGetPr(SLOT_ARG)[0];
        } else {
            bus = (int_T)mxGetPr(SLOT_ARG)[0];
            slot = (int_T)mxGetPr(SLOT_ARG)[1];
        }
        // look for the PCI-Device
        if (rl32eGetPCIInfoAtSlot(0x186C,(unsigned short)devId,(slot & 0xff) | ((bus & 0xff)<< 8),&pciinfo)) {
            sprintf(msg,"%s (bus %d, slot %d): board not present",devName, bus, slot );
            ssSetErrorStatus(S,msg);
            return;
        }
    }

    // show Device Information
    // rl32eShowPCIInfo(pciinfo);

    Physical0=(void *)pciinfo.BaseAddress[0];
    Virtual0 = rl32eGetDevicePtr(Physical0, 32, RT_PG_USERREADWRITE);
    base_chipset=(void *)Physical0;
    Physical1=(void *)pciinfo.BaseAddress[2];
    Virtual1 = rl32eGetDevicePtr(Physical1, 128, RT_PG_USERREADWRITE);
    base_adc=(void *)Physical1;
    Physical2=(void *)pciinfo.BaseAddress[4];
    Virtual2 = rl32eGetDevicePtr(Physical2, 128, RT_PG_USERREADWRITE);
    base_altera=(void *)Physical2;

    ssSetIWorkValue(S, BASE_CHIPSET_I_IND, (int_T)base_chipset);
    ssSetIWorkValue(S, BASE_ADC_I_IND, (int_T)base_adc);
    ssSetIWorkValue(S, BASE_ALTERA_I_IND, (int_T)base_altera);

    base_altera[0x1B] = 0x10101010;                                  // set x4 mode, reset counters
    base_altera[0x1B] = 0x0;                                         // set x4 mode, enable counters
    irc_shadow = 0;

//    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
//        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
//
//        mode = 0;
//        irc_shadow |= (mxGetPr(FILTER_ARG)[i] && 0x01) << 8 * channel;
//        switch ((uint_T)mxGetPr(INDEX_ARG)[i]) {
//			case 0:    mode = 0x00; break;
//			case 1:    mode = 0x20; break;
//			case 2:    mode = 0x30; break;
//			case 3:    mode = 0x40; break;
//			case 4:    mode = 0x50; break;
//			case 5:    mode = 0x60; break;
//			case 6:    mode = 0x0C; break;
//			case 7:    mode = 0x08; break;
//		}
//        irc_shadow |= mode << 8 * channel;
//     }
//    base_altera[0x1B] = irc_shadow;

#endif

}

static void mdlOutputs(SimStruct *S, int_T tid)
{

#ifndef MATLAB_MEX_FILE

    int_T i, n, channel, irc;
    volatile int *base_altera;
    real_T *y;

    base_altera = (void *) ssGetIWorkValue(S, BASE_ALTERA_I_IND);

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
        y=ssGetOutputPortSignal(S,i);

        y[0]=(real_T) base_altera[0x1C+channel];
    }

#endif

}

static void mdlTerminate(SimStruct *S)
{
}


#ifdef MATLAB_MEX_FILE  /* Is this file being compiled as a MEX-file? */
#include "simulink.c"   /* Mex glue */
#else
#include "cg_sfun.h"    /* Code generation glue */
#endif


