/* $Revision:  $ $Date:  $ */
/* pwmhsmf624.c - xPC Target, non-inlined S-function driver for Humusoft MF624 PWM section */
/* Copyright (c) 2003-2006 by Humusoft s.r.o. All Rights Reserved. */

#define S_FUNCTION_LEVEL 2
#define S_FUNCTION_NAME  pwmhsmf624

#include 	<stddef.h>
#include 	<stdlib.h>
#include 	<math.h>

#include 	"simstruc.h"

#ifdef 		MATLAB_MEX_FILE
#include 	"mex.h"
#else
#include 	<windows.h>
#include 	"io_xpcimport.h"
#include    "pci_xpcimport.h"
#endif

/* Input Arguments */
#define NUM_PARAMS             (4)
#define SLOT_ARG               (ssGetSFcnParam(S,0))
#define DEV_ARG                (ssGetSFcnParam(S,1))
#define CHANNEL_ARG            (ssGetSFcnParam(S,2))
#define SAMPLE_TIME_PARAM      (ssGetSFcnParam(S,3))

/* Convert S Function Parameters to Varibles */

#define SLOT                   ((uint_T) mxGetPr(SLOT_ARG)[0])
#define DEVICE                 ((uint_T) mxGetPr(DEV_ARG)[0])
#define SAMPLE_TIME            ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[0])
#define SAMPLE_OFFSET          ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[1])
#define SAMP_TIME_IND          (0)

#define NO_I_WORKS             (3)
#define BASE_CHIPSET_I_IND     (0)
#define BASE_ADC_I_IND         (1)
#define BASE_ALTERA_I_IND      (2)

#define NO_R_WORKS             (0)

static char_T msg[256];



/*====================*
 * S-function methods *
 *====================*/

static void mdlCheckParameters(SimStruct *S)
{
}

static void mdlInitializeSizes(SimStruct *S)
{
    uint_T i;

#ifndef MATLAB_MEX_FILE
#include "io_xpcimport.c"
#include "pci_xpcimport.c"
#endif

    ssSetNumSFcnParams(S, NUM_PARAMS);
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 2*mxGetNumberOfElements(CHANNEL_ARG))) return;

    for (i=0;i<2*mxGetNumberOfElements(CHANNEL_ARG);i++) {
        ssSetInputPortWidth(S, i, 1);
    }

    if (!ssSetNumOutputPorts(S, 0)) return;

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, NO_R_WORKS);
    ssSetNumIWork(S, NO_I_WORKS);
    ssSetNumPWork(S, 0);

    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetSFcnParamNotTunable(S,0);
    ssSetSFcnParamNotTunable(S,1);
    ssSetSFcnParamNotTunable(S,2);
    ssSetSFcnParamNotTunable(S,3);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE | SS_OPTION_PLACE_ASAP);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, SAMPLE_TIME);
    ssSetOffsetTime(S, 0, SAMPLE_OFFSET);
}


#define MDL_START
static void mdlStart(SimStruct *S)
{

#ifndef MATLAB_MEX_FILE
    uint_T i, channel;
    PCIDeviceInfo pciinfo;
    void *Physical2, *Physical1, *Physical0;
    void *Virtual2, *Virtual1, *Virtual0;
    volatile unsigned int *base_chipset;
    volatile unsigned short *base_adc;
    volatile unsigned int *base_altera;
    char devName[20];
    int  devId;

    switch ((int_T)mxGetPr(DEV_ARG)[0]) {
        case 1:
            strcpy(devName,"Humusoft MF624");
            devId=0x0624;
            break;
         }

    if ((int_T)mxGetPr(SLOT_ARG)[0]<0) {
        // look for the PCI-Device
        if (rl32eGetPCIInfo(0x186C,(unsigned short)devId,&pciinfo)) {
            sprintf(msg,"%s: board not present", devName);
            ssSetErrorStatus(S,msg);
            return;
        }
    } else {
        int_T bus, slot;
        if (mxGetN(SLOT_ARG) == 1) {
            bus  = 0;
            slot = (int_T)mxGetPr(SLOT_ARG)[0];
        } else {
            bus = (int_T)mxGetPr(SLOT_ARG)[0];
            slot = (int_T)mxGetPr(SLOT_ARG)[1];
        }
        // look for the PCI-Device
        if (rl32eGetPCIInfoAtSlot(0x186C,(unsigned short)devId,(slot & 0xff) | ((bus & 0xff)<< 8),&pciinfo)) {
            sprintf(msg,"%s (bus %d, slot %d): board not present",devName, bus, slot );
            ssSetErrorStatus(S,msg);
            return;
        }
    }

    // show Device Information
    // rl32eShowPCIInfo(pciinfo);

    Physical0=(void *)pciinfo.BaseAddress[0];
    Virtual0 = rl32eGetDevicePtr(Physical0, 32, RT_PG_USERREADWRITE);
    base_chipset=(void *)Physical0;
    Physical1=(void *)pciinfo.BaseAddress[2];
    Virtual1 = rl32eGetDevicePtr(Physical1, 128, RT_PG_USERREADWRITE);
    base_adc=(void *)Physical1;
    Physical2=(void *)pciinfo.BaseAddress[4];
    Virtual2 = rl32eGetDevicePtr(Physical2, 128, RT_PG_USERREADWRITE);
    base_altera=(void *)Physical2;

    ssSetIWorkValue(S, BASE_CHIPSET_I_IND, (int_T)base_chipset);
    ssSetIWorkValue(S, BASE_ADC_I_IND, (int_T)base_adc);
    ssSetIWorkValue(S, BASE_ALTERA_I_IND, (int_T)base_altera);

/* Set initial conditions */
    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
      channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;

      base_altera[0x18] = 0x02 << (6*channel);        // stop counter
	  base_altera[0x00+0x04*channel] = 0x2E;          // set counter mode
	  base_altera[0x18] = 0x28 << (6*channel);        // reset counter and reset toggle output
	  base_altera[0x01+0x04*channel] = 0;             // reset Load A register
	  base_altera[0x02+0x04*channel] = 0;             // reset Load B register
	  base_altera[0x18] = 0x01 << (6*channel);        // start counter
    }

#endif /* MATLAB_MEX_FILE */

}

static void mdlOutputs(SimStruct *S, int_T tid)
{
#ifndef MATLAB_MEX_FILE

    InputRealPtrsType uPtrs_freq, uPtrs_duty;
    uint_T i, channel;
    volatile unsigned int *base_altera;
    double freq, duty;
    double T;
    unsigned T1, T2, mode;

    base_altera  = (void *) ssGetIWorkValue(S, BASE_ALTERA_I_IND);

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
        uPtrs_freq = ssGetInputPortRealSignalPtrs(S,2*i);
        uPtrs_duty = ssGetInputPortRealSignalPtrs(S,2*i+1);

        freq = *uPtrs_freq[0];
        duty = *uPtrs_duty[0];

        freq = max(freq, 0.012);              // limit inputs
        duty = max(duty, 0);
        duty = min(duty, 1);

        T=50000000/freq;
        T=floor(T + 0.5);
        T1=max(1,floor(T*duty + 0.5));
        T2=max(1,floor(T)-T1);


        base_altera[0x01+0x04*channel] = T1-1;
        base_altera[0x02+0x04*channel] = T2-1;
        mode=0x0E;
        if (duty == 0) mode=0x2E;            // force output low
        if (duty == 1) mode=0x3E;            // force output high
        base_altera[0x04*channel] = mode;
    }

#endif /* MATLAB_MEX_FILE */

}


static void mdlTerminate(SimStruct *S)
{
#ifndef MATLAB_MEX_FILE
    uint_T i, channel;
    volatile unsigned int *base_altera;

    base_altera  = (void *) ssGetIWorkValue(S, BASE_ALTERA_I_IND);

/* Set final outputs to zero */
    for (i=0;i < mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel = (uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
        base_altera[0x04*channel] = 0x2E;             // force output low
    }

#endif /* MATLAB_MEX_FILE */
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
